 /**----------------------------------------------------
 * File:      Editor.java
 * Package:   at.hoertler.svgeditor
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 *
 */
package at.hoertler.svgeditor;

import at.hoertler.svgeditor.action.ActionManager;
import at.hoertler.svgeditor.common.KeyInputManager;
import at.hoertler.svgeditor.data.ElementManager;
import at.hoertler.svgeditor.display.controls.Canvas;
import at.hoertler.svgeditor.display.menubar.MenuBarManager;
import at.hoertler.svgeditor.display.toolbar.ToolBarManager;
import at.hoertler.svgeditor.event.ActionPerformedEvent;
import at.hoertler.svgeditor.event.ElementManagerStateEvent;
import at.hoertler.svgeditor.event.ElementManagerStateListener;
import at.hoertler.svgeditor.options.OptionManager;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;


public class Editor implements ElementManagerStateListener, WindowListener {

  private JFrame parent = null;

  /**
   *
   * @return the parent
   */
  public JFrame getParent() {
    return parent;
  }
  private ToolBarManager toolBarManager = null;
  private MenuBarManager menuBarManager = null;
  private OptionManager optionManager = null;
  private Canvas canvas = null;
  private ElementManager elementManager = null;
  private JScrollPane scrollPane = null;
  private JPanel mainPane = null;

  /**
   * Method init.
   *
   * @param parent   JFrame
   * @param fileName String
   */
  public void init(JFrame parent, String fileName) {
    this.parent = parent;
    this.parent.addWindowListener(this);
    try {
      for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
        if ("Nimbus".equals(info.getName())) {
          UIManager.setLookAndFeel(info.getClassName());
          break;
        }
      }
    } catch (Exception e) {
    }

    //this.parent.getContentPane().setLayout(new BorderLayout());
    this.mainPane = new JPanel(new BorderLayout());
    this.parent.setContentPane(this.mainPane);

    this.optionManager = new OptionManager();
    
    this.elementManager = new ElementManager(this.getOptionManager());
    
    ActionManager.addActionPerformedListener(this.elementManager);
    this.getOptionManager().setElementManager(elementManager);
    //this.elementManager.setElements(SVGImporter.Import());
    this.menuBarManager = new MenuBarManager(this);

    this.elementManager.addElementManagerStateListener(menuBarManager);
    this.elementManager.addElementManagerStateListener(this);
    this.parent.setJMenuBar(this.menuBarManager.getMenuBar());
    

    this.toolBarManager = new ToolBarManager(this);
    this.parent.getContentPane().add(this.toolBarManager.getToolBar(),
                                     BorderLayout.LINE_START);
    
    
    this.canvas = new Canvas(this, this.toolBarManager, this.optionManager);
    ActionManager.addActionPerformedListener(this.canvas);
    this.canvas.addCanvasStateListener(menuBarManager);
    //this.canvas.addKeyListener(new KeyInputManager(this.optionManager, this.actionManager));
    this.scrollPane = new JScrollPane(this.canvas,
                                      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    this.scrollPane.setViewportView(this.canvas);
    //this.canvas.setSizes(this.scrollPane.getSize());
    //this.scrollPane.add(this.canvas);
    //this.scrollPane.SCROLLBARS_ALWAYS;
    //this.scrollPane.set
    //this.scrollPane.setOpaque(true);
    this.parent.getContentPane().add(this.scrollPane, BorderLayout.CENTER);
    

    new KeyInputManager(this.optionManager);
    this.parent.setVisible(true);

    this.scrollPane.addComponentListener(new ComponentAdapter() {
      public void componentResized(ComponentEvent e) {
        JScrollPane c = (JScrollPane) e.getComponent();
        canvas.setSize(new Dimension(c.getWidth() - c.getVerticalScrollBar().
                getWidth(),
                                     c.getHeight() - c.getHorizontalScrollBar().
                getHeight()));
        canvas.setPreferredSize(new Dimension(c.getWidth() - c.
                getVerticalScrollBar().getWidth(),
                                              c.getHeight() - c.
                getHorizontalScrollBar().getHeight()));
      }
    });

    this.scrollPane.resize(this.scrollPane.getWidth() + 1, this.scrollPane.
            getHeight() + 1);
    
    
    if(fileName!="")
      this.elementManager.openFile(new File(fileName));
  }

  /**
   *
   * @return the toolBarManager
   */
  public ToolBarManager getToolBarManager() {
    return toolBarManager;
  }

  /**
   *
   * @return the menuBarManager
   */
  public MenuBarManager getMenuBarManager() {
    return menuBarManager;
  }

  /**
   *
   * @return the ElementContainer object
   */
  public ElementManager getElementManager() {
    return elementManager;
  }

  /**
   * @return the optionManager
   */
  public OptionManager getOptionManager() {
    return optionManager;
  }

  /**
   *
   * @return the Canvas object
   */
  public Canvas getCanvas() {
    return canvas;
  }

  @Override
  public void elementManagerStateChanged(ElementManagerStateEvent e) {
    switch(e.getType()) {
      case openedImage:
        this.parent.setTitle(Reader.getPropertyValueAsString(
            Reader.ConfigProperty.applicationName));
        if(e.isState()) {
          this.parent.setTitle(this.parent.getTitle() + " - " + (String)e.getValue());
        }
        break;
    }
  }

  @Override
  public void windowOpened(WindowEvent e) { }

  @Override
  public void windowClosing(WindowEvent e) {
    ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
            ActionPerformedEvent.Action.closeWindow));
    
  }

  @Override
  public void windowClosed(WindowEvent e) { 
  int i = 0;
  }

  @Override
  public void windowIconified(WindowEvent e) { }

  @Override
  public void windowDeiconified(WindowEvent e) { }

  @Override
  public void windowActivated(WindowEvent e) { }

  @Override
  public void windowDeactivated(WindowEvent e) { }

}
