 /**----------------------------------------------------
 * File:      Reader.java
 * Package:   at.hoertler.svgeditor
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 *
 */
package at.hoertler.svgeditor;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;


public class Reader {

  private static Properties ConfigProperties;
  private static Properties StringProperties;

  /**
   * This is an interface for the propertie enumerations.
   */
  private interface ConfigValue {
  }

  /**
   * This enumeration represents the values that can be loaded from the config
   * file.
   */
  public static enum ConfigProperty
          implements ConfigValue {

    version, applicationName, applicationWidth,
    applicationHeight, logFile, logLevel, editorStringsFile;
  }

  /**
   * This enumeration represents the values that can be loaded from the config
   * file.
   */
  public static enum StringProperty
          implements ConfigValue {

    labelFileMenu, 
      labelNew, labelOpen, labelLastOpened, labelSave, labelSaveAs, labelInsertImage, labelPrint, labelQuit, 
    labelEditMenu, 
      labelUndo, labelRedo, labelDelete, labelCut, labelCopy, labelPaste, labelDuplicate, labelTakePicture, labelGroup, labelUngroup, labelSelectAll, labelInvertSelection, labelDeselect, labelBringToFront, labelSendToBack,  
    select, selectAll, selectArea, drawLine, drawArrow, drawCircle, drawEllipse, drawRect, drawRoundedRect, drawRoundSideRect, drawPoly, drawText,
    chooseLineColor, chooseFillColor, chooseLineStrength, changeZoomFactor
    , changeGridWideness, labelAlignment, labelMagGrid, labelShowMagGrid, labelHideMagGrid,
    labelLeaveGroup,
    newFile_SaveFirst_Title, newFile_SaveFirst_Text,
    saveError_Title, saveError_NoElements_Text
    , drawTriangle;
  }

  /**
   * This enumeration represents the values that can be loaded from the config
   * file.
   */
  public static enum ReaderType {

    ConfigReader, StringReader;
  }

  /**
   * This method sets the path to the configuration file.
   *
   * @param path   The path to the configuration file.
   * @param reader ReaderType
   */
  public static void setConfigurationFile(final ReaderType reader, final String path) {
    Properties properties = null;

    properties = new Properties();
    try {
      properties.load(SVGEditor.class.getResourceAsStream(path));

    } catch (IOException ex) {
      System.err.
              println("IOException occured in Reader.setConfigurationFile(" + reader + "): " + ex.
              toString() + " Path: " + path);
    } catch (NullPointerException ex) {
      System.err.
              println("NullPointerException occured in Reader.setConfigurationFile(" + reader + "): " + ex.
              toString() + " Path: " + path);
    } catch (Exception ex) {
      System.err.
              println("Exception occured in Reader.setConfigurationFile(" + reader + "): " + ex.
              toString() + " Path: " + path);
    }

    switch (reader) {
      case ConfigReader:
        ConfigProperties = properties;
        break;
      case StringReader:
        StringProperties = properties;
        break;
    }
    properties = null;

  }

  /**
   * This method gets the string value for the given propertie key.
   *
   * @param propertie ConfigValue
   *
   * @return Property value as string.
   */
  public static String getPropertyValueAsString(final ConfigValue propertie) {
    try {
      if (propertie.getClass().getName().equals(ConfigProperty.class.getName())) {
        return ConfigProperties.getProperty(propertie.toString());
      } else if (propertie.getClass().getName().equals(StringProperty.class.
              getName())) {
        return StringProperties.getProperty(propertie.toString());
      }

    } catch (Exception ex) {
      System.err.
              println("Exception occured in Reader.getPropertyValueAsString(): " + ex.
              toString());
    }
    return "";
  }

  /**
   * This method gets the integer value for the given propertie key.
   *
   * @param propertie ConfigValue
   *
   * @return Property value as integer.
   */
  public static int getPropertyValueAsInt(final ConfigValue propertie) {
    int ret = 0;
    try {
      ret = Integer.parseInt(Reader.getPropertyValueAsString(propertie));
    } catch (Exception ex) {
      System.err.
              println("Exception occured in Reader.getPropertyValueAsInt(): " + ex.
              toString());
      ret = -1;
    }
    return ret;
  }
}
