 /**----------------------------------------------------
 * File:      ImageSelection.java
 * Package:   at.hoertler.svgeditor.data
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 * The ImageSelection represents the Clipboard-Object
 * which contains image, file, string.
 */
package at.hoertler.svgeditor.data;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;


public class ImageSelection
        implements Transferable {

  BufferedImage image;
  String svg = "";
  ArrayList<File> fileList = new ArrayList<File>();
 
  public ImageSelection(BufferedImage i) {
    this.image = i;
    //this.fileList = null;
    this.svg = "";
  }
  
  public ImageSelection(String s) {
    this.image = null;
    //this.file = null;
    this.svg = s;
  }
          
  public ImageSelection(BufferedImage i, String s) {
    this.image = i;
    //this.file = null;
    this.svg = s;
  }
  public ImageSelection(File f, String s) {
    this.image = null;
    this.fileList.add(f);
    this.svg = s;
  }
  public ImageSelection(BufferedImage i, File f, String s) {
    this.image = i;
    this.fileList.add(f);
    this.svg = s;
  }

  public Object getTransferData(DataFlavor flavor)
          throws UnsupportedFlavorException,
                 IOException {
    if (flavor.equals(DataFlavor.stringFlavor) && this.svg != null)
      return this.svg;
    
    if (flavor.equals(DataFlavor.imageFlavor) && this.image != null)
      return this.image;
    
    if (flavor.equals(DataFlavor.javaFileListFlavor) && this.fileList != null)
      return this.fileList;
    
    throw new UnsupportedFlavorException(flavor);
  }

  public DataFlavor[] getTransferDataFlavors() {
    ArrayList<DataFlavor> al = new ArrayList<DataFlavor>();
    if(this.image!=null)
      al.add(DataFlavor.imageFlavor);
    if(this.svg != null && this.svg != "")
      al.add(DataFlavor.stringFlavor);
    if(this.fileList != null && !this.fileList.isEmpty())
      al.add(DataFlavor.javaFileListFlavor);
    DataFlavor[] flavors = new DataFlavor[al.size()];
    return al.toArray(flavors);
    /*DataFlavor[] flavors = new DataFlavor[4];
    flavors[2] = DataFlavor.imageFlavor;
    flavors[1] = DataFlavor.stringFlavor;
    flavors[0] = DataFlavor.javaFileListFlavor;

    return flavors;*/
  }

  public boolean isDataFlavorSupported(DataFlavor flavor) {
    DataFlavor[] flavors = getTransferDataFlavors();
    for (int i = 0; i < flavors.length; i++)
      if (flavor.equals(flavors[ i]))
        return true;

    return false;
  }
}
