/**----------------------------------------------------
 * File:      GridWidenessOptionPopUp.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Popup used for setting the wideness of the magnetic grid.
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.event.OptionChangedEvent.Option;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import javax.swing.Icon;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class GridWidenessOptionPopUp extends OptionPopUp {
    
    public GridWidenessOptionPopUp(Option o, Object v)    {
        super(o,v);
    }

    @Override
    protected void init() {
        this.removeAll();
        this.setLayout(new FlowLayout(FlowLayout.CENTER));

        SpinnerNumberModel model = new SpinnerNumberModel((int)this.getValue(Option.gridWideness), 1, 50, 1); 
        JSpinner size = new JSpinner(model);
        size.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                int grid = (int)((JSpinner)e.getSource()).getModel().getValue();
                if(grid == 0)
                  grid = 1;
                values.put(Option.gridWideness, grid);
                fireOptionPopUpCommit(false);
            }
        });
        this.add(size);
    }
    

    @Override
    public Icon createIcon() {
        return new Icon(){
            public int getIconHeight() {
                return 16;
            }
            public int getIconWidth() {
                return 16;
            }

            @Override
            public void paintIcon( Component c, Graphics g, int x, int y ) {
                Graphics2D g2 = (Graphics2D)g.create();
                g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
                g2.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
                
                g2.setColor(Color.BLACK);
                // Create Circle
                //Ellipse2D circle = new Ellipse2D.Float(5, 5, 7, 6);
                //g2.draw(circle);
                
                Line2D line = new Line2D.Float(x+2,y+5,x+14,y+5);
                g2.draw(line);
                line = new Line2D.Float(x+2,y+10,x+14,y+10);
                g2.draw(line);
                line = new Line2D.Float(x+5,y+2,x+5,y+14);
                g2.draw(line);
                line = new Line2D.Float(x+10,y+2,x+10,y+14);
                g2.draw(line);
                
                g2.setColor(Color.RED);
                g2.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
                // Create Circle
                Ellipse2D circle = new Ellipse2D.Float(x+2, y+2, 6, 6);
                g2.draw(circle);
            }

        };
    }
}
