/**----------------------------------------------------
 * File:      ToolBarButton.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * This abstract ToolBarButton is the parent of all ToolBarElements.
 * It can be used to create a new type of ToolBarButtons.
 */
package at.hoertler.svgeditor.display.toolbar;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.Icon;
import javax.swing.JButton;

public abstract class ToolBarButton extends JButton implements MouseListener {

    private boolean focused = false;
    private boolean focusAble = true;
    private ToolBarManager toolBarManager = null;

    public ToolBarButton(String name, ToolBarManager tbm) {
        this(name, name, tbm);
    }

    public ToolBarButton(String name, String toolTipText, ToolBarManager tbm) {
        this.setToolBarManager(tbm);
        this.setName(name);
        this.setToolTipText(toolTipText);
        //this.addMouseListener(this);
        this.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (focusAble) {
                    toolBarManager.setActiveButton((ToolBarButton) e.getSource());
                }
            }
        });
    }

    /**
     * @return If the ToolBarButton is focused.
     */
    public boolean hasFocus() {
        return this.focused;
    }

    /**
     * Sets the focus state of the ToolBarButton.
     *
     * @param focused the focus state to set
     */
    public void setFocus(boolean focused) {
        if (this.isFocusAble()) {
            this.focused = focused;
        }
    }

    /**
     * @return the toolBarManager
     */
    public ToolBarManager getToolBarManager() {
        return toolBarManager;
    }

    /**
     * @param toolBarManager the toolBarManager to set
     */
    public void setToolBarManager(ToolBarManager toolBarManager) {
        this.toolBarManager = toolBarManager;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
    }

    @Override
    public void mousePressed(MouseEvent e) {
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }

    @Override
    public void mouseEntered(MouseEvent e) {
    }

    @Override
    public void mouseExited(MouseEvent e) {
    }

    /**
     * Method creates an icon to be shown.
     *
     * @return Icon to represent the button.
     */
    protected abstract Icon createIcon();

    /**
     * @return the focusAble
     */
    public boolean isFocusAble() {
        return focusAble;
    }

    /**
     * @param focusAble the focusAble to set
     */
    public void setFocusAble(boolean focusAble) {
        this.focusAble = focusAble;
    }
}
