/**----------------------------------------------------
 * File:      ZoomOptionPopUp.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Popup used for zooming.
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.event.OptionChangedEvent.Option;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import javax.swing.Icon;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


public class ZoomOptionPopUp extends OptionPopUp {
    
    public ZoomOptionPopUp(Option o, Object v)    {
        super(o,v);
    }

    @Override
    protected void init() {
        this.removeAll();
        this.setLayout(new FlowLayout(FlowLayout.CENTER));
        JSlider slider = new JSlider(JSlider.HORIZONTAL, 1, 10, (int)this.getValue(Option.zoomFactor));
        slider.setMajorTickSpacing(3);
        slider.setMinorTickSpacing(1);
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                setValue(Option.zoomFactor, ((JSlider)e.getSource()).getValue());
                fireOptionPopUpCommit(false);
            }
        });
        this.add(slider);
    }
    

    @Override
    public Icon createIcon() {
        return new Icon(){
            public int getIconHeight() {
                return 16;
            }
            public int getIconWidth() {
                return 16;
            }

            @Override
            public void paintIcon( Component c, Graphics g, int x, int y ) {
                Graphics2D g2 = (Graphics2D)g;

                g2.setStroke(new BasicStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));

                g2.setColor(Color.BLACK);
                // Create Circle
                Ellipse2D circle = new Ellipse2D.Float(5, 5, 7, 6);
                g2.draw(circle);
                
                Line2D line = new Line2D.Float(12,11,16,16);
                g2.draw(line);
            }

        };
    }
}
