 /**----------------------------------------------------
 * File:      SVGRectangle.java
 * Package:   at.hoertler.svgeditor.svg.elements
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Represents an SVGElement.
 */
package at.hoertler.svgeditor.svg.elements;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.util.Vector;
import javax.swing.Icon;

public class SVGRectangle
        extends SVGElement {

  @Override
  public Point getTranslationPointForHandle(int handleIndex) {
    return this.getHandles().get((handleIndex+2)%4);
  }

  public enum Type { Normal, RoundedRectangle, RoundSidedRectangle };
  private Type type = Type.Normal;
    private int rXY = 20;
  
  public SVGRectangle() {
    this.setFillcolorSetable(true);
    this.getHandles().add(new Point(-1,-1));
    this.getHandles().add(new Point(-1,-1));
    this.getHandles().add(new Point(-1,-1));
    this.getHandles().add(new Point(-1,-1));
    this.setTextSetable(true);
  }

  @Override
  public SVGElement getInstance() {
    SVGRectangle r = new SVGRectangle();
    r.setType(this.getType());
    return r;
  }

  /**
   * @return the type
   */
  public Type getType() {
    return type;
  }

  /**
   * @param type the type to set
   */
  public void setType(Type type) {
    this.type = type;
    if(this.type == Type.RoundSidedRectangle) {
      this.standardTextMargin = 30;
      this.updateTextBoundingBox();
    }
  }

  @Override
  public void paint(Graphics g) {
    this.paint(g, 1);
  }

  @Override
  public void paint(Graphics g, int zoomFactor) {
    Vector<Point> handles = this.getHandles();
    //Point a = calcZoom(handles.get(0),zoomFactor);
    //Point b = calcZoom(handles.get(1),zoomFactor);
    this.setGraphics((Graphics2D) g);
    Rectangle2D r = this.createRectangle(zoomFactor);
    this.getGraphics().setStroke(getStroke(zoomFactor));

    
    switch(this.getType()) {
      case Normal:
        if (this.isFillcolorSetable() && this.getFillColor() != null) {
          this.getGraphics().setColor(this.getFillColor());
          this.getGraphics().fill(r);
        }
        if(this.getLineStrength()!=-1) {
          this.getGraphics().setColor(this.getLineColor());
          this.getGraphics().draw(r);
        }
        break;
      case RoundedRectangle:
        RoundRectangle2D rnd = new RoundRectangle2D.Float((int)r.getX(), (int)r.getY(), (int)r.getWidth(), (int)r.getHeight(), rXY*zoomFactor, rXY*zoomFactor);
        if (this.isFillcolorSetable() && this.getFillColor() != null) {
          this.getGraphics().setColor(this.getFillColor());
          this.getGraphics().fill(rnd);
        }
        if(this.getLineStrength()!=-1) {
          this.getGraphics().setColor(this.getLineColor());
          this.getGraphics().draw(rnd);
        }
        break;
      case RoundSidedRectangle:
        Shape[] shapes = createRoundSidedRectangleShapes(r);
        CubicCurve2D side1 = (CubicCurve2D)shapes[0];
        CubicCurve2D side2 = (CubicCurve2D)shapes[1];
        Line2D line1 = (Line2D)shapes[2];
        Line2D line2 = (Line2D)shapes[3];
        if (this.isFillcolorSetable() && this.getFillColor() != null) {
          this.getGraphics().setColor(this.getFillColor());
          this.getGraphics().fillRect((int)(line1.getP1().getX()), (int)(line1.getP1().getY()), (int)(line2.getP2().getX()-line1.getP1().getX()), (int)(line2.getP2().getY()-line1.getP1().getY()));
          this.getGraphics().fill(side1);
          this.getGraphics().fill(side2);
        }
        if(this.getLineStrength()!=-1) {
          this.getGraphics().setColor(this.getLineColor());
          this.getGraphics().draw(line1);
          this.getGraphics().draw(line2);
          this.getGraphics().draw(side1);
          this.getGraphics().draw(side2);
        }
        break;
    }

    this.renderText(zoomFactor);
  }
  
  private Shape[] createRoundSidedRectangleShapes(Rectangle2D r) {
    CubicCurve2D side1;
    CubicCurve2D side2;
    Line2D line1;
    Line2D line2;
    if(r.getWidth()>r.getHeight()) {
      side1 = new CubicCurve2D.Float((int)(r.getX()+(r.getHeight()/2)), (int)r.getY(),
                                     (int)(r.getX()),                   (int)(r.getY()),
                                     (int)(r.getX()),                   (int)(r.getY() + (r.getHeight())),
                                     (int)(r.getX()+(r.getHeight()/2)), (int)(r.getY() + r.getHeight()));
      side2 = new CubicCurve2D.Float((int)(r.getX()+r.getWidth()-(r.getHeight()/2)),  (int)r.getY(),
                                     (int)(r.getX()+r.getWidth()),                    (int)(r.getY()),
                                     (int)(r.getX()+r.getWidth()),                    (int)(r.getY() + (r.getHeight())),
                                     (int)(r.getX()+r.getWidth()-(r.getHeight()/2)),  (int)(r.getY() + r.getHeight()));
      line1 = new Line2D.Float((int)side1.getP1().getX(), (int)r.getY(), (int)side2.getP1().getX(), (int)r.getY());
      line2 = new Line2D.Float((int)side1.getP2().getX(), (int)(r.getY()+r.getHeight()), (int)side2.getP2().getX(), (int)(r.getY()+r.getHeight()));
    } else {
      side1 = new CubicCurve2D.Float((int)(r.getX()),                (int)(r.getY()+(r.getWidth()/2)),
                                     (int)(r.getX()),                (int)(r.getY()),
                                     (int)(r.getX() + r.getWidth()), (int)(r.getY()),
                                     (int)(r.getX() + r.getWidth()), (int)(r.getY()+(r.getWidth()/2)));
      side2 = new CubicCurve2D.Float((int)(r.getX()),                (int)(r.getY() + r.getHeight() - (r.getWidth()/2)),
                                     (int)(r.getX()),                (int)(r.getY() + r.getHeight()),
                                     (int)(r.getX()+r.getWidth()),   (int)(r.getY() + r.getHeight()),
                                     (int)(r.getX()+r.getWidth()),   (int)(r.getY() + r.getHeight() - (r.getWidth()/2)));
      line1 = new Line2D.Float((int)r.getX(), (int)side1.getP1().getY(), (int)r.getX(), (int)side2.getP1().getY());
      line2 = new Line2D.Float((int)(r.getX()+r.getWidth()), (int)side1.getP1().getY(), (int)(r.getX()+r.getWidth()), (int)side2.getP1().getY());;
    }
    Shape[] shapes = new Shape[4];
    shapes[0] = side1;
    shapes[1] = side2;
    shapes[2] = line1;
    shapes[3] = line2;
    return shapes;
  }

  @Override
  public Icon getIcon() {
    return new Icon() {
      public int getIconHeight() {
        return 16;
      }

      public int getIconWidth() {
        return 16;
      }

      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        
        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setColor(Color.BLACK);
        g2.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));
        if(getType() == Type.RoundedRectangle) 
          g2.drawRoundRect(x + 1, y + 4, 12, 8, 4, 4);
        else if(getType() == Type.Normal) 
          g2.drawRect(x + 1, y + 4, 12, 8);
        else if(getType() == Type.RoundSidedRectangle) {
          Point p1 = new Point(x+4,y+4);
          Point p2 = new Point(x+11,y+4);
          Point p3 = new Point(x+4,y+12);
          Point p4 = new Point(x+11,y+12);
          int off = 4;
          g2.drawLine(p1.x,p1.y,p2.x,p2.y);
          g2.drawLine(p3.x,p3.y,p4.x,p4.y);
          CubicCurve2D c1 = new CubicCurve2D.Float(p1.x,p1.y, 
                                                   p1.x-off,p1.y, 
                                                   p3.x-off,p3.y, 
                                                   p3.x,p3.y);
          CubicCurve2D c2 = new CubicCurve2D.Float(p2.x,p2.y, 
                                                   p2.x+off,p2.y, 
                                                   p4.x+off,p4.y, 
                                                   p4.x,p4.y);
          g2.draw(c1);
          g2.draw(c2);
        }
      }
    };
  }

  @Override
  public void moveHandle(int handleIndex, Point p, MouseEvent e) {
    if (this.isNormalMode()) {
      return;
    }
    this.getHandles().get(handleIndex).translate(p.x, p.y);
    recalculateHandles(handleIndex,p);
    this.updateBoundingRect(this.getHandles());
  }
  
  private void recalculateHandles(int index, Point translation) {
    Vector<Point> handles = this.getHandles();
    switch(index) {
      case 0:
        if(translation.x!=0)
          handles.get(3).translate(translation.x,0);
        if(translation.y!=0)
          handles.get(1).translate(0, translation.y);
        break;
      case 1:
        if(translation.x!=0)
          handles.get(2).translate(translation.x,0);
        if(translation.y!=0)
          handles.get(0).translate(0, translation.y);
        break;
      case 2:
        if(translation.x!=0)
          handles.get(1).translate(translation.x,0);
        if(translation.y!=0)
          handles.get(3).translate(0, translation.y);
        break;
      case 3:
        if(translation.x!=0)
          handles.get(0).translate(translation.x,0);
        if(translation.y!=0)
          handles.get(2).translate(0, translation.y);
        break;
    }
  }
  
  private void reorderHandles() {
    Point h0 = new Point(minX(this.getHandles(), 1), minY(this.getHandles(), 1));
    Point h1 = new Point(maxX(this.getHandles(), 1), minY(this.getHandles(), 1));
    Point h2 = new Point(maxX(this.getHandles(), 1), maxY(this.getHandles(), 1));
    Point h3 = new Point(minX(this.getHandles(), 1), maxY(this.getHandles(), 1));
    this.getHandles().get(0).setLocation(h0);
    this.getHandles().get(1).setLocation(h1);
    this.getHandles().get(2).setLocation(h2);
    this.getHandles().get(3).setLocation(h3);
  }

  @Override
  public boolean validate() {
    if(this.getHandles().isEmpty()) return false;
    this.reorderHandles();
    if (this.getHandles().get(0).equals(this.getHandles().get(1)) || this.getHandles().get(0).equals(this.getHandles().get(3)))
      return false;
    return true;
  }

  @Override
  public String exportToSVG() {
    Rectangle2D r = this.createRectangle(1);
    StringBuilder sb = new StringBuilder();
    Color c = this.getLineColor();
    if(this.getType() == Type.RoundSidedRectangle) {
      Shape[] shapes = createRoundSidedRectangleShapes(r);
      CubicCurve2D side1 = (CubicCurve2D)shapes[0];
      CubicCurve2D side2 = (CubicCurve2D)shapes[1];
      Line2D line1 = (Line2D)shapes[2];
      Line2D line2 = (Line2D)shapes[3];
      sb.append("\t<g id=\"SVGRoundSideRectangle\">\n");
      
      sb.append(this.exportTextToSVG());
      
      sb.append("\t\t");
      //<path stroke="red" fill="none" d="m100,200c0,-100 150,-100 150,0"/>
      sb.append("<path ");
      sb.append("stroke=\"rgb("
              + c.getRed() + ","
              + c.getGreen() + ","
              + c.getBlue() + ")\" ");
      c = this.getFillColor();
      if (c.getAlpha() != 0) {
        sb.append("fill=\"rgb("
                + c.getRed() + ","
                + c.getGreen() + ","
                + c.getBlue() + ")\" ");
      } else {
        sb.append("fill=\"none\" ");
      }
      sb.append("stroke-width=\"" + this.getExportLineStrength() + "\" ");
      sb.append("d=\"");
      sb.append("M" + (int)side1.getP1().getX() + "," + (int)side1.getP1().getY());
      sb.append("C" + (int)side1.getCtrlP1().getX() + "," + (int)side1.getCtrlP1().getY() + " "
                    + (int)side1.getCtrlP2().getX() + "," + (int)side1.getCtrlP2().getY() + " "
                    + (int)side1.getP2().getX() + "," + (int)side1.getP2().getY());
      sb.append("L" + (int)line2.getP2().getX() + ","+ (int)line2.getP2().getY());
      sb.append("C" + (int)side2.getCtrlP2().getX() + "," + (int)side2.getCtrlP2().getY() + " "
                    + (int)side2.getCtrlP1().getX() + "," + (int)side2.getCtrlP1().getY() + " "
                    + (int)side2.getP1().getX() + "," + (int)side2.getP1().getY());
      sb.append("L" + (int)line1.getP1().getX() + ","+ (int)line1.getP1().getY());
      sb.append("Z\" ");
      
    } else if(this.getType() == Type.Normal || this.getType() == Type.RoundedRectangle) {
      sb.append("\t");
      sb.append("<g id=\"" + this.getClass().getSimpleName() + "\">\n");
      sb.append(this.exportTextToSVG());
      sb.append("\t\t");
      sb.append("<rect ");
      sb.append("x=\"" + (int)r.getX() + "\" "
              + "y=\"" + (int)r.getY() + "\" ");
      sb.append("height=\"" + (int)r.getHeight() + "\" "
              + "width=\"" + (int)r.getWidth() + "\" ");
      sb.append("stroke=\"rgb("
              + c.getRed() + ","
              + c.getGreen() + ","
              + c.getBlue() + ")\" ");
      c = this.getFillColor();
      if (c.getAlpha() != 0) {
        sb.append("fill=\"rgb("
                + c.getRed() + ","
                + c.getGreen() + ","
                + c.getBlue() + ")\" ");
      } else {
        sb.append("fill=\"none\" ");
      }
      sb.append("stroke-width=\"" + this.getExportLineStrength() + "\" ");
      if(this.getType() == Type.RoundedRectangle) {
        sb.append("rx=\"" + this.rXY + "\" ");
        sb.append("ry=\"" + this.rXY + "\" ");
      }
    }
    sb.append("/>");
    sb.append("\t</g>\n");
    return sb.toString();
  }

  @Override
  public void handleMousePressed(MouseEvent e) {
    if(!this.isInitialCreateMode()) return;
    
    this.getHandles().set(0, e.getPoint());
    this.getHandles().set(1, e.getPoint());
    this.getHandles().set(2, e.getPoint());
    this.getHandles().set(3, e.getPoint());
    this.updateBoundingRect(this.getHandles());
  }

  @Override
  public void handleMouseDragged(MouseEvent e) {
    if(!this.isInitialCreateMode()) return;
    
    this.getHandles().set(2, e.getPoint());
    this.updateBoundingRect(this.getHandles());
  }

  @Override
  public void handleMouseReleased(MouseEvent e) {
    if(!this.isInitialCreateMode()) return;
    
    reorderHandles();
    this.updateBoundingRect(this.getHandles());
    this.setMode(Mode.Edit);
  }

  @Override
  public void handleMouseClicked(MouseEvent e) {
    if(!this.isEditMode() && !this.isTextEditMode()) return;

    if(e.getClickCount()>=2) {
      if(this.isTextEditMode())
        this.setMode(Mode.Edit);
      else
        this.setMode(Mode.TextEdit);
    }
  }

  @Override
  public boolean hit(Rectangle rect) {
    Vector<Point> handles = this.getHandles();

    Line2D l = new Line2D.Double(handles.get(0), handles.get(1));
    return createRectangle(1).intersects(rect);
  }

  private Rectangle2D createRectangle(int zoomFactor) {
    Point xy = new Point(minX(this.getHandles(), zoomFactor),minY(this.getHandles(), zoomFactor));
    Dimension d = new Dimension(maxX(this.getHandles(), zoomFactor) - xy.x,
                                maxY(this.getHandles(), zoomFactor) - xy.y);
    return new Rectangle2D.Float(xy.x, xy.y, d.width, d.height);
  }
}
