 /**----------------------------------------------------
 * File:      SVGText.java
 * Package:   at.hoertler.svgeditor.svg.elements
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Represents an SVGElement.
 */
package at.hoertler.svgeditor.svg.elements;

import at.hoertler.svgeditor.common.KeyInputManager;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.font.GlyphVector;
import java.awt.font.TextAttribute;
import java.text.AttributedString;
import javax.swing.Icon;

public class SVGText
        extends SVGElement {

  private Dimension textDimension = new Dimension(0,0);
  
  public SVGText() {
    this.getHandles().add(new Point(-1,-1));
    this.getHandles().add(new Point(-1,-1));
    this.setTextSetable(true);
    this.setFillcolorSetable(true);
  }
  
  @Override
  public SVGElement getInstance() {
    SVGText t =  new SVGText();
    //t.setMode(Mode.Normal);
    return t;
  }

  @Override
  public void paint(Graphics g) {
    this.paint(g, 1);
  }

  @Override
  public void paint(Graphics g, int zoomFactor) {
    this.setGraphics((Graphics2D)g);
     this.calcTextDimension();
    

    //if(!this.getText().isEmpty()) {
      Point xy = calcZoom(this.getBoundingBox().getLocation(), zoomFactor);
      Dimension d = new Dimension(this.getBoundingBox().width*zoomFactor, this.getBoundingBox().height*zoomFactor);
      Rectangle rect = new Rectangle(xy,d);

      if (this.isFillcolorSetable() && this.getFillColor() != null) {
        this.getGraphics().setColor(this.getFillColor());
        this.getGraphics().fill(rect);
      }
      if(this.getLineStrength()!=-1) {
        this.getGraphics().setColor(this.getLineColor());
        this.getGraphics().setStroke(getStroke(zoomFactor));
        this.getGraphics().draw(rect);
      }
      this.renderText(zoomFactor);
    //}
    //this.renderText(zoomFactor);
  }
  
  @Override
  public void setText(String text) {
    super.setText(text);
    this.calcTextDimension();
    
  }
  
  private void calcTextDimension() {
    String text = this.getText();
    if(this.getGraphics()==null) return;
    if(text.isEmpty()) text+=" ";
    if(text.endsWith("\n"))
      text+=" ";

    String[] lines = text.split("\n");
    int height = this.getTextSize()*lines.length + 2*this.getTextMargin();
    int width = 0;
    AttributedString as = this.getAttributedString(text, 1);
    Font f = (Font)as.getIterator().getAttribute(TextAttribute.FONT);

    for(int i = 0; i<lines.length; i++) {
      GlyphVector vec = f.createGlyphVector(this.getGraphics().getFontRenderContext(), lines[i]);
      width = Math.max(width, (int)Math.ceil(vec.getVisualBounds().getBounds().width));
    }
    this.textDimension.setSize(width, height);
    if(this.getHandles().size()>=2) {
      Point p = null;
      switch(this.getTextAlignment()) {
        case left:
          p = this.getHandles().get(0);
          this.getHandles().get(1).setLocation(p.x+this.textDimension.width+2*this.getTextMargin(), 
                                               this.getHandles().get(0).y+height);
          break;
        case center:
          p = (Point)this.getHandles().get(0).clone();
          p.x += (this.getHandles().get(1).x - p.x)/2;
          this.getHandles().get(0).setLocation(p.x-(this.textDimension.width/2+this.getTextMargin()), p.y);
          this.getHandles().get(1).setLocation(p.x+(this.textDimension.width/2+this.getTextMargin()), 
                                               this.getHandles().get(0).y+height);
          break;
        case right: 
          p = this.getHandles().get(1);
          this.getHandles().get(0).setLocation(p.x-(this.textDimension.width+2*this.getTextMargin()), 
                                               this.getHandles().get(0).y);
          this.getHandles().get(1).setLocation(p.x, 
                                               this.getHandles().get(0).y+height);

          break;
      }
      this.updateBoundingRect(this.getHandles());
    }
  }

  @Override
  public boolean hit(Rectangle rect) {
    return this.getBoundingBox().intersects(rect);
  }

  @Override
  public Icon getIcon() {
    return new Icon() {
      public int getIconHeight() {
        return 16;
      }

      public int getIconWidth() {
        return 16;
      }

      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        Graphics2D g2 = (Graphics2D) g;
        /*g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);*/
        g2.setColor(Color.BLACK);
        AttributedString as = new AttributedString("T");
        as.addAttribute(TextAttribute.FOREGROUND, Color.BLACK);
        Font plainFont = new Font(Font.SERIF, Font.PLAIN, 16);
        as.addAttribute(TextAttribute.FONT, plainFont.deriveFont(Font.BOLD));
        g2.drawString(as.getIterator(), x+3, y+14);
      }
    };
  }

  @Override
  public void moveHandle(int handleIndex, Point p, MouseEvent e) { }

  @Override
  public boolean validate() {
    return (!this.getText().isEmpty() || this.isTextEditMode());
    //return true;//(!this.getText().isEmpty());// || this.isExtendedCreate());
  }

  @Override
  public String exportToSVG() {
    StringBuilder sb = new StringBuilder();
    /*
    <text fill="blue" font-size="50" font-family="Verdana" y="150" x="50">Hello, out there</text>
    <text y="10">
        <tspan x="10">tspan line 1</tspan>
        <tspan x="10" dy="15">tspan line 2</tspan>
        <tspan x="10" dy="15">tspan line 3</tspan>
    </text>
                */
    
    sb.append("\t");
    sb.append("<g id=\"" + this.getClass().getSimpleName() + "\">\n");
    
    
    Rectangle r = this.getBoundingBox();
    Color c = this.getLineColor();
    sb.append("\t\t");
    sb.append("<rect ");
      sb.append("x=\"" + (int)r.getX() + "\" "
              + "y=\"" + (int)r.getY() + "\" ");
      sb.append("height=\"" + (int)r.getHeight() + "\" "
              + "width=\"" + (int)r.getWidth() + "\" ");
      sb.append("stroke=\"rgb("
              + c.getRed() + ","
              + c.getGreen() + ","
              + c.getBlue() + ")\" ");
      c = this.getFillColor();
      if (c.getAlpha() != 0) {
        sb.append("fill=\"rgb("
                + c.getRed() + ","
                + c.getGreen() + ","
                + c.getBlue() + ")\" ");
      } else {
        sb.append("fill=\"none\" ");
      }
      sb.append("stroke-width=\"" + this.getExportLineStrength() + "\" ");
      sb.append("/>\n");
    

    sb.append(this.exportTextToSVG());
    
    sb.append("\t");
    sb.append("</g>\n");
    return sb.toString();
  }

  @Override
  public void handleMousePressed(MouseEvent e) {
    if(!this.isInitialCreateMode()) return;
    this.setLineStrength(-1);
    this.textDimension.height = this.getTextSize();
    this.textDimension.width = 0;
    //this.getHandles().get(0).setLocation(e.getX()+this.textDimension.width*2/3, e.getY()-this.textDimension.height*2/3);
    //this.getHandles().get(0).setLocation(e.getX(), e.getY());
    //this.getHandles().get(1).setLocation(e.getX()+this.textDimension.width, e.getY()+this.textDimension.height);
    int margLeft = this.getTextMargin();
    int margRight = this.getTextMargin();
    //if(this.getTextAlignment().equals(OptionManager.Alignment.right))
      //marg *= -1;
    int height = this.getTextSize() + 2*this.getTextMargin();
    switch(this.getTextAlignment()) {
        case left:
          margLeft = 0;
          margRight *= 2;
          break;
        case center:
          margRight *= 2;
          break;
        case right: 
          margLeft *= 2;
          //margRight = 0;
          break;
      }
    this.getHandles().get(0).setLocation(e.getX()-margLeft, e.getY()-height/2);//this.textDimension.height*2/3);
    this.getHandles().get(1).setLocation(e.getX()+margRight, e.getY()+height/2);
    //this.getHandles().get(0).setLocation(e.getX()-this.getTextMargin()+this.getTextMargin(), e.getY()-this.getTextMargin()*2);//this.textDimension.height*2/3);
    //this.getHandles().get(1).setLocation(e.getX()+this.getTextMargin()+this.getTextMargin(), e.getY()+this.textDimension.height+this.getTextMargin());
    this.calcTextDimension();
    
/*
    if(this.getHandles().size()>=2) {
      Point p = this.getHandles().get(0);
      this.getHandles().get(1).setLocation(p.x+this.textDimension.width+2*this.getTextMargin(), p.y+this.textDimension.height+2*this.getTextMargin());
      this.updateBoundingRect(this.getHandles());
    }*/
  }

  @Override
  public void handleMouseDragged(MouseEvent e) { }

  @Override
  public void handleMouseReleased(MouseEvent e) {
    if(!this.isInitialCreateMode()) return;
    this.setMode(Mode.Edit);
    this.setMode(Mode.TextEdit);
  }

  @Override
  public void handleMouseClicked(MouseEvent e) {
    if(e.getClickCount()>=2) {
      if(this.isTextEditMode())
        this.setMode(Mode.Edit);
      else
        this.setMode(Mode.TextEdit);
    }
  }
  
  @Override
  public Point getTranslationPointForHandle(int handleIndex) {
    return this.getHandles().get((handleIndex+1)%2);
  }
  
  @Override
  public void setMode(Mode mode) {
    Mode m = this.getMode();
    super.setMode(mode);
    switch(m) {
      case Normal:
        switch(mode) {
          case Edit:
            KeyInputManager.registerKeyInputReceiver(this);
            break;
        }
        break;
      case Edit:
        switch(mode) {
          case Normal:
            KeyInputManager.removeKeyInputReceiver(this);
            break;
        }
    }
    
  }
  
  @Override
  public Cursor getInitialCreateCursor() {
      return new Cursor(Cursor.TEXT_CURSOR);
  }
}
